/*********************************************************************/
/*                   Solomon Systech Limited                         */
/*********************************************************************/
/* Program Name :main.c
/* Description : Define SD routines for programming Zilog Z8 Encore! eZ8F642c
/*               8-bit MCU to drive SSD1926 DVK board
/*				 This souce code provides a reference design for customers who intend
/*				 to use any 8-bit MCU to control SSD1926 DVK board
/*				 The SD routine here is to communicate with the hardware of SD card holder
/*				 It also set the clock and power of SD card
/*				 
/* Author : Aik Hong (SSL Singapore FAE)
/*********************************************************************/

/*********************************************************************/
/*                        Change Log                                 */
/*********************************************************************/
/* 13 March 2008: Initial Revision
/* Modifications :
/* 13 Mar 08  	Version 1 stamped
/* 13 Mar 08  	1.0 : Include routine to do file system as to extract file info from SD card
/* 9  Apr 08  	Version 2 stamped
/* 7  May 08  	Version 3 stamped
/* 7  May 08  	3.0	: Increase the time delay in reading RCA from SD card
/* 7  Aug 08    Fix a bug, so that we do not have to send ACMD 41 to SD twice
/*********************************************************************/

#include "io.h"
#include "defines.h"
#include "sdcmd.h"
#include "lcd.h"
#include "jpegcmd.h"
#include "ssd192X.h"

#include <TinyFS\tff.h>
#include <string.h>

UINT16 SD_RCA;

#define NULL		(void *)0

// return 0 if no card insert otherwise
UINT8 Card_Detect()
{
	char card;
	UINT32 card_1, card_2;
	UINT16 card_w;

//	SD_Clck_Ctrl();
	card_w = SSD_REGRW(0x1126);
	card_2 = SSD_REGRL(0x1124);
	card = SSD_REGRB(0x1126);

	card_1 = card_2 = SSD_REGRL(0x1124);
	while((card & 0x00020000) == 0x00000000)   // wait for card to stable before reading in value
	{
		card_1 = card_2 =  SSD_REGRL(0x1124);
	}
//	card = SSD_REGRB(0x1126);

//	return(0);
	return(card_2 & 0x00010000);
}

// SD clock control
// this will turn on the SD clock
void SD_Clck_Ctrl(char value)
{
    //SSD_REGWB(0x112D, 0x02); 						// set SD clock divisor 4 (20MHz for 80MHz)
	if( value == 0)
	{
		SSD_REGWB(0x112D, 0x02);		//SSD_REGWB(0x112D, 0x40);   
	}
	else
	{
		SSD_REGWB(0x112D, 0x40);
	}

    SSD_REGWB(0x112C, SSD_REGRB(0x112C)|0x01); // enable INTernal clock

    while(!(SSD_REGRB(0x112C)&0x02)); // wait SD clock to become stable

    SSD_REGWB(0x112C, SSD_REGRB(0x112C)|0x04); // enable SD clock
}

// Turn off SD clock
void SD_Clck_Off(void)
{
    SSD_REGWB(0x112C, SSD_REGRB(0x112C)&~0x04); // enable SD clock
}

// SD bus power
void SD_Power_On(void)
{
	char pwr;

    if(SSD_REGRB(0x1143)&0x01) SSD_REGWB(0x1129, 0x0F);
	pwr = SSD_REGRB(0x1129);

}

// SD host data width and INTerrupt
void SD_Host_Ctrl(void)
{
    SSD_REGWB(0x1128, 0x02);
    SSD_REGWB(0x1135, SSD_REGRB(0x1135)|0x01);
    SSD_REGWB(0x1137, SSD_REGRB(0x1137)|0x01);
    SSD_REGWB(0x1139, SSD_REGRB(0x1139)|0x01);
    SSD_REGWB(0x113B, SSD_REGRB(0x113B)|0x01);
}

UINT8 SD_Send_CMD(UINT8 cmd, UINT32 cmd_argu, UINT8 cmd_resp_type)
{

    if (!(SSD_REGRL(0x1124)&0x01))
    {
        SSD_REGWB(0x1130, 0x01); //clear command complete INTerrupt
        SSD_REGWB(0x1132, 0x01); //clear command timeout error
        SSD_REGWB(0x110B, cmd_argu>>24); //arg data
        SSD_REGWB(0x110A, (cmd_argu&0xFF0000)>>16); //arg data
        SSD_REGWB(0x1109, (cmd_argu&0xFF00)>>8); //arg data
        SSD_REGWB(0x1108, cmd_argu&0xFF); //arg data
        SSD_REGWB(0x110E, cmd_resp_type); //set response type
        SSD_REGWB(0x110F, cmd); //command index
        return 1;
    } else return 0;
}

void SD_Send_appCMD(UINT8 cmd, UINT32 cmd_argu, UINT8 cmd_resp_type)
{
	//UINT8 cnt =0;

    while(!SD_Send_CMD(CMD55, SD_RCA<<16, wResp_CMD));
    delay(500);
    while(!SD_Send_CMD(cmd, cmd_argu, cmd_resp_type));	
	while(SSD_REGRB(0x1132)|0xFE == 0xFF);
}

void SD_init(void) {

    UINT32 response_voltage, add, state;
	UINT8 a, b;
    SD_RCA = 0x0000; // Card address set to 0x0000

	SSD_REGWB(0x1134, SSD_REGRB(0x1134)|0x01); // cmd complete enable
    SSD_REGWB(0x1136, SSD_REGRB(0x1136)|0x01); // timeout error enable
    SSD_REGWB(0x1138, SSD_REGRB(0x1138)|0x01); // cmd complete signal INTerrupt
    SSD_REGWB(0x113A, SSD_REGRB(0x113A)|0x01); // timeout error signal INTerrupt
    SSD_REGWB(0x1130, 0x01); //clear previous INTerrupt
    SSD_REGWB(0x1132, 0x23); //clear previous INTerrupt
    SSD_REGWB(0x1128, 0x02); //data width 4bits


    while(!SD_Send_CMD(CMD0, 0x00, NORM_CMD));
	while(!SSD_REGRB(0x1130)&0x01);   		// check to ensure command is complete
	
    SD_Send_appCMD(ACMD41, 0x00, wResp_CMD); // ask card send OCR back
    delay(1000);	//1000
	response_voltage = (SSD_REGRL(0x1110));

reset:
 	SD_Send_appCMD(ACMD41, response_voltage, wResp_CMD); // ask card send OCR back
    delay(1500);	// 1500
	state = SSD_REGRL(0x1110) & 0x80000000;	// read back the status
	if ( state != 0x80000000 )
		goto reset;		// no response from card

    
	while(!SD_Send_CMD(CMD2, 0x00, r2_resp|cmd_crc_chk));
	delay(1500);	
    while(SSD_REGRB(0x1132)|0xFE == 0xFF);
    while(!SSD_REGRB(0x1130)&0x01);
    
	while(!SD_Send_CMD(CMD3, 0x00, r6_resp|cmd_check));		// get RCA 
    while(SSD_REGRB(0x1132)|0xFE == 0xFF);
    while(!SSD_REGRB(0x1130)&0x01);

    //SD_RCA = SSD_REGRW(0x1112);
	add =  SSD_REGRL(0x1110);
	SD_RCA = add>>16;
}

void SD_Test(void) {

	UINT8 i,iob[16], temp;
	UINT32 state;
	
    SSD_REGWB(0x112F, 0x07);		    // reset all parameters
    while(SSD_REGRB(0x112F)&0x07);
    SD_Power_On();
    SD_Clck_Ctrl(0);

	//Card_Detect();

    SD_init();

    Select_card();
    
    SD_Send_appCMD(ACMD6, 0x02, r1_resp|cmd_check); 	//set SD card to use data width 4bits
	//state = SSD_REGRL(0x1110);

	while(!SD_Send_CMD(CMD16, 512, r1_resp|cmd_check)); //set block len cmd
 	while(!SSD_REGRB(0x1130)&0x01); 
   	while(SSD_REGRB(0x1132)|0xFE == 0xFF);
}

void SD_send_csd(void)
{
    UINT32 argument;
    argument = SD_RCA << 16;
    while(!SD_Send_CMD(CMD55, argument, R2_Resp_CMD));
    // put the code to read response from card
}


INT8 write_sdcard(void) {

	UINT8 gImage_luma[] = {0x55, 0x45, 0x4C,0x4C, 0x4F};

    return 0;
}


